
/* 
Five Finger Code Finder (C)2017, 2018 Carl Smith
Unlocks car doors with DeBruijn minimal length sequence

NOTE:  I am not very experienced with Arduino or C++ code and I am sure there
are many improvements to be made to this code, as small as it is.

With delay=100 it does about 291 keypresses per minute and finishes the entire 
sequence in 10 minutes and 45 seconds.

NOTE: There is a chance that this thing will reprogram your door code.
Before experimenting be sure you know the unchangable factory code and
how to reset your code.

*/


// include the library code:
#include <LiquidCrystal.h>
#include <avr/pgmspace.h>

// initialize the library with the numbers of the interface pins
LiquidCrystal lcd(12, 11, 5, 4, 3, 2);


/* NOTE: I had to add four 1's to the end of the following sequence to get it to 
   test 99991, 99911, 99111, and 91111. The Wikipedia program that generates the 
   sequence treats it like a circle - all the codes are not tested unless you are
   going around continuously.  It really takes 3129 presses to test 3125 codes since
   the first four presses do not complete a five digit code.
*/ 

const PROGMEM char debruijn5of5[] = {
1, 1, 1, 1, 1, 3, 1, 1, 1, 1, 5, 1, 1, 1, 1, 7, 1, 1, 1, 1, 9, 1, 1, 1, 3, 3, 1, 1, 1, 3,
5, 1, 1, 1, 3, 7, 1, 1, 1, 3, 9, 1, 1, 1, 5, 3, 1, 1, 1, 5, 5, 1, 1, 1, 5, 7, 1, 1, 1, 5,
9, 1, 1, 1, 7, 3, 1, 1, 1, 7, 5, 1, 1, 1, 7, 7, 1, 1, 1, 7, 9, 1, 1, 1, 9, 3, 1, 1, 1, 9,
5, 1, 1, 1, 9, 7, 1, 1, 1, 9, 9, 1, 1, 3, 1, 3, 1, 1, 3, 1, 5, 1, 1, 3, 1, 7, 1, 1, 3, 1,
9, 1, 1, 3, 3, 3, 1, 1, 3, 3, 5, 1, 1, 3, 3, 7, 1, 1, 3, 3, 9, 1, 1, 3, 5, 3, 1, 1, 3, 5,
5, 1, 1, 3, 5, 7, 1, 1, 3, 5, 9, 1, 1, 3, 7, 3, 1, 1, 3, 7, 5, 1, 1, 3, 7, 7, 1, 1, 3, 7,
9, 1, 1, 3, 9, 3, 1, 1, 3, 9, 5, 1, 1, 3, 9, 7, 1, 1, 3, 9, 9, 1, 1, 5, 1, 3, 1, 1, 5, 1,
5, 1, 1, 5, 1, 7, 1, 1, 5, 1, 9, 1, 1, 5, 3, 3, 1, 1, 5, 3, 5, 1, 1, 5, 3, 7, 1, 1, 5, 3,
9, 1, 1, 5, 5, 3, 1, 1, 5, 5, 5, 1, 1, 5, 5, 7, 1, 1, 5, 5, 9, 1, 1, 5, 7, 3, 1, 1, 5, 7,
5, 1, 1, 5, 7, 7, 1, 1, 5, 7, 9, 1, 1, 5, 9, 3, 1, 1, 5, 9, 5, 1, 1, 5, 9, 7, 1, 1, 5, 9,
9, 1, 1, 7, 1, 3, 1, 1, 7, 1, 5, 1, 1, 7, 1, 7, 1, 1, 7, 1, 9, 1, 1, 7, 3, 3, 1, 1, 7, 3,
5, 1, 1, 7, 3, 7, 1, 1, 7, 3, 9, 1, 1, 7, 5, 3, 1, 1, 7, 5, 5, 1, 1, 7, 5, 7, 1, 1, 7, 5,
9, 1, 1, 7, 7, 3, 1, 1, 7, 7, 5, 1, 1, 7, 7, 7, 1, 1, 7, 7, 9, 1, 1, 7, 9, 3, 1, 1, 7, 9,
5, 1, 1, 7, 9, 7, 1, 1, 7, 9, 9, 1, 1, 9, 1, 3, 1, 1, 9, 1, 5, 1, 1, 9, 1, 7, 1, 1, 9, 1,
9, 1, 1, 9, 3, 3, 1, 1, 9, 3, 5, 1, 1, 9, 3, 7, 1, 1, 9, 3, 9, 1, 1, 9, 5, 3, 1, 1, 9, 5,
5, 1, 1, 9, 5, 7, 1, 1, 9, 5, 9, 1, 1, 9, 7, 3, 1, 1, 9, 7, 5, 1, 1, 9, 7, 7, 1, 1, 9, 7,
9, 1, 1, 9, 9, 3, 1, 1, 9, 9, 5, 1, 1, 9, 9, 7, 1, 1, 9, 9, 9, 1, 3, 1, 3, 3, 1, 3, 1, 3,
5, 1, 3, 1, 3, 7, 1, 3, 1, 3, 9, 1, 3, 1, 5, 3, 1, 3, 1, 5, 5, 1, 3, 1, 5, 7, 1, 3, 1, 5,
9, 1, 3, 1, 7, 3, 1, 3, 1, 7, 5, 1, 3, 1, 7, 7, 1, 3, 1, 7, 9, 1, 3, 1, 9, 3, 1, 3, 1, 9,
5, 1, 3, 1, 9, 7, 1, 3, 1, 9, 9, 1, 3, 3, 1, 5, 1, 3, 3, 1, 7, 1, 3, 3, 1, 9, 1, 3, 3, 3,
3, 1, 3, 3, 3, 5, 1, 3, 3, 3, 7, 1, 3, 3, 3, 9, 1, 3, 3, 5, 3, 1, 3, 3, 5, 5, 1, 3, 3, 5,
7, 1, 3, 3, 5, 9, 1, 3, 3, 7, 3, 1, 3, 3, 7, 5, 1, 3, 3, 7, 7, 1, 3, 3, 7, 9, 1, 3, 3, 9,
3, 1, 3, 3, 9, 5, 1, 3, 3, 9, 7, 1, 3, 3, 9, 9, 1, 3, 5, 1, 5, 1, 3, 5, 1, 7, 1, 3, 5, 1,
9, 1, 3, 5, 3, 3, 1, 3, 5, 3, 5, 1, 3, 5, 3, 7, 1, 3, 5, 3, 9, 1, 3, 5, 5, 3, 1, 3, 5, 5,
5, 1, 3, 5, 5, 7, 1, 3, 5, 5, 9, 1, 3, 5, 7, 3, 1, 3, 5, 7, 5, 1, 3, 5, 7, 7, 1, 3, 5, 7,
9, 1, 3, 5, 9, 3, 1, 3, 5, 9, 5, 1, 3, 5, 9, 7, 1, 3, 5, 9, 9, 1, 3, 7, 1, 5, 1, 3, 7, 1,
7, 1, 3, 7, 1, 9, 1, 3, 7, 3, 3, 1, 3, 7, 3, 5, 1, 3, 7, 3, 7, 1, 3, 7, 3, 9, 1, 3, 7, 5,
3, 1, 3, 7, 5, 5, 1, 3, 7, 5, 7, 1, 3, 7, 5, 9, 1, 3, 7, 7, 3, 1, 3, 7, 7, 5, 1, 3, 7, 7,
7, 1, 3, 7, 7, 9, 1, 3, 7, 9, 3, 1, 3, 7, 9, 5, 1, 3, 7, 9, 7, 1, 3, 7, 9, 9, 1, 3, 9, 1,
5, 1, 3, 9, 1, 7, 1, 3, 9, 1, 9, 1, 3, 9, 3, 3, 1, 3, 9, 3, 5, 1, 3, 9, 3, 7, 1, 3, 9, 3,
9, 1, 3, 9, 5, 3, 1, 3, 9, 5, 5, 1, 3, 9, 5, 7, 1, 3, 9, 5, 9, 1, 3, 9, 7, 3, 1, 3, 9, 7,
5, 1, 3, 9, 7, 7, 1, 3, 9, 7, 9, 1, 3, 9, 9, 3, 1, 3, 9, 9, 5, 1, 3, 9, 9, 7, 1, 3, 9, 9,
9, 1, 5, 1, 5, 3, 1, 5, 1, 5, 5, 1, 5, 1, 5, 7, 1, 5, 1, 5, 9, 1, 5, 1, 7, 3, 1, 5, 1, 7,
5, 1, 5, 1, 7, 7, 1, 5, 1, 7, 9, 1, 5, 1, 9, 3, 1, 5, 1, 9, 5, 1, 5, 1, 9, 7, 1, 5, 1, 9,
9, 1, 5, 3, 1, 7, 1, 5, 3, 1, 9, 1, 5, 3, 3, 3, 1, 5, 3, 3, 5, 1, 5, 3, 3, 7, 1, 5, 3, 3,
9, 1, 5, 3, 5, 3, 1, 5, 3, 5, 5, 1, 5, 3, 5, 7, 1, 5, 3, 5, 9, 1, 5, 3, 7, 3, 1, 5, 3, 7,
5, 1, 5, 3, 7, 7, 1, 5, 3, 7, 9, 1, 5, 3, 9, 3, 1, 5, 3, 9, 5, 1, 5, 3, 9, 7, 1, 5, 3, 9,
9, 1, 5, 5, 1, 7, 1, 5, 5, 1, 9, 1, 5, 5, 3, 3, 1, 5, 5, 3, 5, 1, 5, 5, 3, 7, 1, 5, 5, 3,
9, 1, 5, 5, 5, 3, 1, 5, 5, 5, 5, 1, 5, 5, 5, 7, 1, 5, 5, 5, 9, 1, 5, 5, 7, 3, 1, 5, 5, 7,
5, 1, 5, 5, 7, 7, 1, 5, 5, 7, 9, 1, 5, 5, 9, 3, 1, 5, 5, 9, 5, 1, 5, 5, 9, 7, 1, 5, 5, 9,
9, 1, 5, 7, 1, 7, 1, 5, 7, 1, 9, 1, 5, 7, 3, 3, 1, 5, 7, 3, 5, 1, 5, 7, 3, 7, 1, 5, 7, 3,
9, 1, 5, 7, 5, 3, 1, 5, 7, 5, 5, 1, 5, 7, 5, 7, 1, 5, 7, 5, 9, 1, 5, 7, 7, 3, 1, 5, 7, 7,
5, 1, 5, 7, 7, 7, 1, 5, 7, 7, 9, 1, 5, 7, 9, 3, 1, 5, 7, 9, 5, 1, 5, 7, 9, 7, 1, 5, 7, 9,
9, 1, 5, 9, 1, 7, 1, 5, 9, 1, 9, 1, 5, 9, 3, 3, 1, 5, 9, 3, 5, 1, 5, 9, 3, 7, 1, 5, 9, 3,
9, 1, 5, 9, 5, 3, 1, 5, 9, 5, 5, 1, 5, 9, 5, 7, 1, 5, 9, 5, 9, 1, 5, 9, 7, 3, 1, 5, 9, 7,
5, 1, 5, 9, 7, 7, 1, 5, 9, 7, 9, 1, 5, 9, 9, 3, 1, 5, 9, 9, 5, 1, 5, 9, 9, 7, 1, 5, 9, 9,
9, 1, 7, 1, 7, 3, 1, 7, 1, 7, 5, 1, 7, 1, 7, 7, 1, 7, 1, 7, 9, 1, 7, 1, 9, 3, 1, 7, 1, 9, 
5, 1, 7, 1, 9, 7, 1, 7, 1, 9, 9, 1, 7, 3, 1, 9, 1, 7, 3, 3, 3, 1, 7, 3, 3, 5, 1, 7, 3, 3, 
7, 1, 7, 3, 3, 9, 1, 7, 3, 5, 3, 1, 7, 3, 5, 5, 1, 7, 3, 5, 7, 1, 7, 3, 5, 9, 1, 7, 3, 7,
3, 1, 7, 3, 7, 5, 1, 7, 3, 7, 7, 1, 7, 3, 7, 9, 1, 7, 3, 9, 3, 1, 7, 3, 9, 5, 1, 7, 3, 9,
7, 1, 7, 3, 9, 9, 1, 7, 5, 1, 9, 1, 7, 5, 3, 3, 1, 7, 5, 3, 5, 1, 7, 5, 3, 7, 1, 7, 5, 3,
9, 1, 7, 5, 5, 3, 1, 7, 5, 5, 5, 1, 7, 5, 5, 7, 1, 7, 5, 5, 9, 1, 7, 5, 7, 3, 1, 7, 5, 7,
5, 1, 7, 5, 7, 7, 1, 7, 5, 7, 9, 1, 7, 5, 9, 3, 1, 7, 5, 9, 5, 1, 7, 5, 9, 7, 1, 7, 5, 9,
9, 1, 7, 7, 1, 9, 1, 7, 7, 3, 3, 1, 7, 7, 3, 5, 1, 7, 7, 3, 7, 1, 7, 7, 3, 9, 1, 7, 7, 5,
3, 1, 7, 7, 5, 5, 1, 7, 7, 5, 7, 1, 7, 7, 5, 9, 1, 7, 7, 7, 3, 1, 7, 7, 7, 5, 1, 7, 7, 7,
7, 1, 7, 7, 7, 9, 1, 7, 7, 9, 3, 1, 7, 7, 9, 5, 1, 7, 7, 9, 7, 1, 7, 7, 9, 9, 1, 7, 9, 1,
9, 1, 7, 9, 3, 3, 1, 7, 9, 3, 5, 1, 7, 9, 3, 7, 1, 7, 9, 3, 9, 1, 7, 9, 5, 3, 1, 7, 9, 5,
5, 1, 7, 9, 5, 7, 1, 7, 9, 5, 9, 1, 7, 9, 7, 3, 1, 7, 9, 7, 5, 1, 7, 9, 7, 7, 1, 7, 9, 7,
9, 1, 7, 9, 9, 3, 1, 7, 9, 9, 5, 1, 7, 9, 9, 7, 1, 7, 9, 9, 9, 1, 9, 1, 9, 3, 1, 9, 1, 9,
5, 1, 9, 1, 9, 7, 1, 9, 1, 9, 9, 1, 9, 3, 3, 3, 1, 9, 3, 3, 5, 1, 9, 3, 3, 7, 1, 9, 3, 3,
9, 1, 9, 3, 5, 3, 1, 9, 3, 5, 5, 1, 9, 3, 5, 7, 1, 9, 3, 5, 9, 1, 9, 3, 7, 3, 1, 9, 3, 7,
5, 1, 9, 3, 7, 7, 1, 9, 3, 7, 9, 1, 9, 3, 9, 3, 1, 9, 3, 9, 5, 1, 9, 3, 9, 7, 1, 9, 3, 9,
9, 1, 9, 5, 3, 3, 1, 9, 5, 3, 5, 1, 9, 5, 3, 7, 1, 9, 5, 3, 9, 1, 9, 5, 5, 3, 1, 9, 5, 5,
5, 1, 9, 5, 5, 7, 1, 9, 5, 5, 9, 1, 9, 5, 7, 3, 1, 9, 5, 7, 5, 1, 9, 5, 7, 7, 1, 9, 5, 7,
9, 1, 9, 5, 9, 3, 1, 9, 5, 9, 5, 1, 9, 5, 9, 7, 1, 9, 5, 9, 9, 1, 9, 7, 3, 3, 1, 9, 7, 3,
5, 1, 9, 7, 3, 7, 1, 9, 7, 3, 9, 1, 9, 7, 5, 3, 1, 9, 7, 5, 5, 1, 9, 7, 5, 7, 1, 9, 7, 5,
9, 1, 9, 7, 7, 3, 1, 9, 7, 7, 5, 1, 9, 7, 7, 7, 1, 9, 7, 7, 9, 1, 9, 7, 9, 3, 1, 9, 7, 9,
5, 1, 9, 7, 9, 7, 1, 9, 7, 9, 9, 1, 9, 9, 3, 3, 1, 9, 9, 3, 5, 1, 9, 9, 3, 7, 1, 9, 9, 3,
9, 1, 9, 9, 5, 3, 1, 9, 9, 5, 5, 1, 9, 9, 5, 7, 1, 9, 9, 5, 9, 1, 9, 9, 7, 3, 1, 9, 9, 7,
5, 1, 9, 9, 7, 7, 1, 9, 9, 7, 9, 1, 9, 9, 9, 3, 1, 9, 9, 9, 5, 1, 9, 9, 9, 7, 1, 9, 9, 9,
9, 3, 3, 3, 3, 3, 5, 3, 3, 3, 3, 7, 3, 3, 3, 3, 9, 3, 3, 3, 5, 5, 3, 3, 3, 5, 7, 3, 3, 3,
5, 9, 3, 3, 3, 7, 5, 3, 3, 3, 7, 7, 3, 3, 3, 7, 9, 3, 3, 3, 9, 5, 3, 3, 3, 9, 7, 3, 3, 3,
9, 9, 3, 3, 5, 3, 5, 3, 3, 5, 3, 7, 3, 3, 5, 3, 9, 3, 3, 5, 5, 5, 3, 3, 5, 5, 7, 3, 3, 5,
5, 9, 3, 3, 5, 7, 5, 3, 3, 5, 7, 7, 3, 3, 5, 7, 9, 3, 3, 5, 9, 5, 3, 3, 5, 9, 7, 3, 3, 5,
9, 9, 3, 3, 7, 3, 5, 3, 3, 7, 3, 7, 3, 3, 7, 3, 9, 3, 3, 7, 5, 5, 3, 3, 7, 5, 7, 3, 3, 7,
5, 9, 3, 3, 7, 7, 5, 3, 3, 7, 7, 7, 3, 3, 7, 7, 9, 3, 3, 7, 9, 5, 3, 3, 7, 9, 7, 3, 3, 7,
9, 9, 3, 3, 9, 3, 5, 3, 3, 9, 3, 7, 3, 3, 9, 3, 9, 3, 3, 9, 5, 5, 3, 3, 9, 5, 7, 3, 3, 9,
5, 9, 3, 3, 9, 7, 5, 3, 3, 9, 7, 7, 3, 3, 9, 7, 9, 3, 3, 9, 9, 5, 3, 3, 9, 9, 7, 3, 3, 9,
9, 9, 3, 5, 3, 5, 5, 3, 5, 3, 5, 7, 3, 5, 3, 5, 9, 3, 5, 3, 7, 5, 3, 5, 3, 7, 7, 3, 5, 3,
7, 9, 3, 5, 3, 9, 5, 3, 5, 3, 9, 7, 3, 5, 3, 9, 9, 3, 5, 5, 3, 7, 3, 5, 5, 3, 9, 3, 5, 5,
5, 5, 3, 5, 5, 5, 7, 3, 5, 5, 5, 9, 3, 5, 5, 7, 5, 3, 5, 5, 7, 7, 3, 5, 5, 7, 9, 3, 5, 5,
9, 5, 3, 5, 5, 9, 7, 3, 5, 5, 9, 9, 3, 5, 7, 3, 7, 3, 5, 7, 3, 9, 3, 5, 7, 5, 5, 3, 5, 7,
5, 7, 3, 5, 7, 5, 9, 3, 5, 7, 7, 5, 3, 5, 7, 7, 7, 3, 5, 7, 7, 9, 3, 5, 7, 9, 5, 3, 5, 7,
9, 7, 3, 5, 7, 9, 9, 3, 5, 9, 3, 7, 3, 5, 9, 3, 9, 3, 5, 9, 5, 5, 3, 5, 9, 5, 7, 3, 5, 9,
5, 9, 3, 5, 9, 7, 5, 3, 5, 9, 7, 7, 3, 5, 9, 7, 9, 3, 5, 9, 9, 5, 3, 5, 9, 9, 7, 3, 5, 9,
9, 9, 3, 7, 3, 7, 5, 3, 7, 3, 7, 7, 3, 7, 3, 7, 9, 3, 7, 3, 9, 5, 3, 7, 3, 9, 7, 3, 7, 3,
9, 9, 3, 7, 5, 3, 9, 3, 7, 5, 5, 5, 3, 7, 5, 5, 7, 3, 7, 5, 5, 9, 3, 7, 5, 7, 5, 3, 7, 5,
7, 7, 3, 7, 5, 7, 9, 3, 7, 5, 9, 5, 3, 7, 5, 9, 7, 3, 7, 5, 9, 9, 3, 7, 7, 3, 9, 3, 7, 7,
5, 5, 3, 7, 7, 5, 7, 3, 7, 7, 5, 9, 3, 7, 7, 7, 5, 3, 7, 7, 7, 7, 3, 7, 7, 7, 9, 3, 7, 7,
9, 5, 3, 7, 7, 9, 7, 3, 7, 7, 9, 9, 3, 7, 9, 3, 9, 3, 7, 9, 5, 5, 3, 7, 9, 5, 7, 3, 7, 9,
5, 9, 3, 7, 9, 7, 5, 3, 7, 9, 7, 7, 3, 7, 9, 7, 9, 3, 7, 9, 9, 5, 3, 7, 9, 9, 7, 3, 7, 9,
9, 9, 3, 9, 3, 9, 5, 3, 9, 3, 9, 7, 3, 9, 3, 9, 9, 3, 9, 5, 5, 5, 3, 9, 5, 5, 7, 3, 9, 5,
5, 9, 3, 9, 5, 7, 5, 3, 9, 5, 7, 7, 3, 9, 5, 7, 9, 3, 9, 5, 9, 5, 3, 9, 5, 9, 7, 3, 9, 5,
9, 9, 3, 9, 7, 5, 5, 3, 9, 7, 5, 7, 3, 9, 7, 5, 9, 3, 9, 7, 7, 5, 3, 9, 7, 7, 7, 3, 9, 7,
7, 9, 3, 9, 7, 9, 5, 3, 9, 7, 9, 7, 3, 9, 7, 9, 9, 3, 9, 9, 5, 5, 3, 9, 9, 5, 7, 3, 9, 9,
5, 9, 3, 9, 9, 7, 5, 3, 9, 9, 7, 7, 3, 9, 9, 7, 9, 3, 9, 9, 9, 5, 3, 9, 9, 9, 7, 3, 9, 9,
9, 9, 5, 5, 5, 5, 5, 7, 5, 5, 5, 5, 9, 5, 5, 5, 7, 7, 5, 5, 5, 7, 9, 5, 5, 5, 9, 7, 5, 5,
5, 9, 9, 5, 5, 7, 5, 7, 5, 5, 7, 5, 9, 5, 5, 7, 7, 7, 5, 5, 7, 7, 9, 5, 5, 7, 9, 7, 5, 5,
7, 9, 9, 5, 5, 9, 5, 7, 5, 5, 9, 5, 9, 5, 5, 9, 7, 7, 5, 5, 9, 7, 9, 5, 5, 9, 9, 7, 5, 5,
9, 9, 9, 5, 7, 5, 7, 7, 5, 7, 5, 7, 9, 5, 7, 5, 9, 7, 5, 7, 5, 9, 9, 5, 7, 7, 5, 9, 5, 7,
7, 7, 7, 5, 7, 7, 7, 9, 5, 7, 7, 9, 7, 5, 7, 7, 9, 9, 5, 7, 9, 5, 9, 5, 7, 9, 7, 7, 5, 7,
9, 7, 9, 5, 7, 9, 9, 7, 5, 7, 9, 9, 9, 5, 9, 5, 9, 7, 5, 9, 5, 9, 9, 5, 9, 7, 7, 7, 5, 9,
7, 7, 9, 5, 9, 7, 9, 7, 5, 9, 7, 9, 9, 5, 9, 9, 7, 7, 5, 9, 9, 7, 9, 5, 9, 9, 9, 7, 5, 9,
9, 9, 9, 7, 7, 7, 7, 7, 9, 7, 7, 7, 9, 9, 7, 7, 9, 7, 9, 7, 7, 9, 9, 9, 7, 9, 7, 9, 9, 7,
9, 9, 9, 9, 9, 1, 1, 1, 1};


const int button1Pin = 14;   //Analog A0 pin
const int button2Pin = 15;   //Analog A1 pin
const int button3Pin = 16;   //Analog A2 pin
const int button4Pin = 17;   //Analog A3 pin
const int button5Pin = 18;   //Analog A4 pin

const int solenoid1Pin = 6;
const int solenoid2Pin = 7;
const int solenoid3Pin = 8;
const int solenoid4Pin = 9;
const int solenoid5Pin = 10;

const int state_Start = 1;
const int state_Run = 2;
const int state_Stop = 3;


int timer = 100;                      // Time in milliseconds between keypresses
int i = 0;                            // loop counter
unsigned char currentdigit;           // hold value read from sequence stored in program memory
String line2 = "                ";    // Start with 16 spaces on second line of LCD


void setup()
{
  // set up the LCD's number of columns and rows and print project name
  lcd.begin(16, 2);
  lcd.print("5FCF-CDS");

  pinMode(button1Pin, INPUT_PULLUP);
  pinMode(button2Pin, INPUT_PULLUP);
  pinMode(button3Pin, INPUT_PULLUP);
  pinMode(button4Pin, INPUT_PULLUP);
  pinMode(button5Pin, INPUT_PULLUP);

  pinMode(solenoid1Pin, OUTPUT);
  pinMode(solenoid2Pin, OUTPUT);
  pinMode(solenoid3Pin, OUTPUT);
  pinMode(solenoid4Pin, OUTPUT);
  pinMode(solenoid5Pin, OUTPUT);

  lcd.setCursor(0, 1);   // Move cursor to first column, second row
  lcd.print("PRESS START");

  while (digitalRead(button1Pin) == HIGH) {}   //Loop until button pressed
  while (digitalRead(button1Pin) == LOW)  {}   //Then wait for release

  // loop through whole DeBruijn sequence
  for (i = 0; i < (sizeof(debruijn5of5)/sizeof(debruijn5of5[0])); i++)
  {
    solenoidsOff();
    if (digitalRead(button1Pin) == LOW) pausemode();   // If necessary pause between solenoid presses
    
    currentdigit = pgm_read_word_near(debruijn5of5 + i);
    
    lcd.setCursor(0, 1);    // Move cursor to first column, second row
    //Drop leftmost digit and add new one to right end by
    //adding new digit to the substring starting with the second digit
    line2 = line2.substring(1) + currentdigit;   
    lcd.print (line2);
    
    lcd.setCursor(12, 0);   // Move cursor to column 13, first row
    lcd.print(i+1);         // Print index of current digit on top row, left end
                            //  but add one since array count starts at zero
    delay (timer);
    solenoidOn(currentdigit);
    delay (timer);          // Slow the universe down so we can see it work
       
  }
  solenoidsOff();           // Sequence is over, shut off solenoids
}


void pausemode()
{
  lcd.setCursor(0, 0);      // Move cursor to first column, first row
  lcd.print("PAUSED  ");
  delay (200);
  while (digitalRead(button1Pin) == LOW) {}   //Button is low when we get here.  Wait for release. 

  solenoidsOff();                             //Don't want solenoids to sit and cook while in pause.
    
  while (digitalRead(button1Pin) == HIGH)     //Now wait for button press again to restart.
    { checkBackForward(); }                   //But check for back and forward buttons while waiting.
     
  lcd.setCursor(0, 0);      // Move cursor to first column, first row
  lcd.print("5FCF-CDS");
}


void checkBackForward()
{
  if (digitalRead(button2Pin) == LOW)   //Back Button Pressed 
  {  
    lcd.setCursor(0, 0);   // Move cursor to first column, first row
    lcd.print("BACK    ");
    delay (200);
    while (digitalRead(button2Pin) == LOW) {}   //Button is low when we get here.  Wait for release. 
    i -= 16;
    if (i < 0) { i = 0; }   
  }

  if (digitalRead(button3Pin) == LOW)   //Forward Button Pressed 
  {  
    lcd.setCursor(0, 0);   // Move cursor to first column, first row
    lcd.print("FORWARD  ");
    delay (200);
    while (digitalRead(button3Pin) == LOW) {}   //Button is low when we get here.  Wait for release. 
    i += 16;  
  }

  line2 = "                ";
  lcd.setCursor(12, 0);  // Move cursor to column 13, first row
  lcd.print(i);          // Print index of current digit on top row, left end
  lcd.print("  ");
}


void solenoidOn (unsigned char sol)
{
  if (sol == 1) { digitalWrite(solenoid1Pin, 1); }
  if (sol == 3) { digitalWrite(solenoid2Pin, 1); }
  if (sol == 5) { digitalWrite(solenoid3Pin, 1); }
  if (sol == 7) { digitalWrite(solenoid4Pin, 1); }
  if (sol == 9) { digitalWrite(solenoid5Pin, 1); }
}


void solenoidsOff()
{
  digitalWrite(solenoid1Pin, 0);
  digitalWrite(solenoid2Pin, 0);
  digitalWrite(solenoid3Pin, 0);
  digitalWrite(solenoid4Pin, 0);
  digitalWrite(solenoid5Pin, 0);
}


void loop(){}


